<?php

namespace Btravel\Toppartner\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Btravel\Toppartner\Http\Requests\ToppartnerRequest;
use Btravel\Toppartner\Interfaces\ToppartnerRepositoryInterface;
use Btravel\Toppartner\Models\Toppartner;

/**
 * Resource controller class for toppartner.
 */
class ToppartnerResourceController extends BaseController
{

    /**
     * Initialize toppartner resource controller.
     *
     * @param type ToppartnerRepositoryInterface $toppartner
     *
     * @return null
     */
    public function __construct(ToppartnerRepositoryInterface $toppartner)
    {
        parent::__construct();
        $this->repository = $toppartner;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Btravel\Toppartner\Repositories\Criteria\ToppartnerResourceCriteria::class);
    }

    /**
     * Display a list of toppartner.
     *
     * @return Response
     */
    public function index(ToppartnerRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Btravel\Toppartner\Repositories\Presenter\ToppartnerPresenter::class)
                ->$function();
        }

        $toppartners = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('toppartner::toppartner.names'))
            ->view('toppartner::toppartner.index', true)
            ->data(compact('toppartners', 'view'))
            ->output();
    }

    /**
     * Display toppartner.
     *
     * @param Request $request
     * @param Model   $toppartner
     *
     * @return Response
     */
    public function show(ToppartnerRequest $request, Toppartner $toppartner)
    {

        if ($toppartner->exists) {
            $view = 'toppartner::toppartner.show';
        } else {
            $view = 'toppartner::toppartner.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('toppartner::toppartner.name'))
            ->data(compact('toppartner'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new toppartner.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ToppartnerRequest $request)
    {

        $toppartner = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('toppartner::toppartner.name')) 
            ->view('toppartner::toppartner.create', true) 
            ->data(compact('toppartner'))
            ->output();
    }

    /**
     * Create new toppartner.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ToppartnerRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $toppartner                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('toppartner::toppartner.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('toppartner/toppartner/' . $toppartner->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/toppartner/toppartner'))
                ->redirect();
        }

    }

    /**
     * Show toppartner for editing.
     *
     * @param Request $request
     * @param Model   $toppartner
     *
     * @return Response
     */
    public function edit(ToppartnerRequest $request, Toppartner $toppartner)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('toppartner::toppartner.name'))
            ->view('toppartner::toppartner.edit', true)
            ->data(compact('toppartner'))
            ->output();
    }

    /**
     * Update the toppartner.
     *
     * @param Request $request
     * @param Model   $toppartner
     *
     * @return Response
     */
    public function update(ToppartnerRequest $request, Toppartner $toppartner)
    {
        try {
            $attributes = $request->all();

            $toppartner->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('toppartner::toppartner.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('toppartner/toppartner/' . $toppartner->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('toppartner/toppartner/' . $toppartner->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the toppartner.
     *
     * @param Model   $toppartner
     *
     * @return Response
     */
    public function destroy(ToppartnerRequest $request, Toppartner $toppartner)
    {
        try {

            $toppartner->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('toppartner::toppartner.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('toppartner/toppartner/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('toppartner/toppartner/' . $toppartner->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple toppartner.
     *
     * @param Model   $toppartner
     *
     * @return Response
     */
    public function delete(ToppartnerRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('toppartner::toppartner.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('toppartner/toppartner'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/toppartner/toppartner'))
                ->redirect();
        }

    }

    /**
     * Restore deleted toppartners.
     *
     * @param Model   $toppartner
     *
     * @return Response
     */
    public function restore(ToppartnerRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('toppartner::toppartner.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/toppartner/toppartner'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/toppartner/toppartner/'))
                ->redirect();
        }

    }

}
